;***********************************************************************************;
;***********************************************************************************;
;
; A wedge to fix bugs in the VIC-20 RS-232 KERNAL routines.
; By Simon Rowe <srowe@mose.org.uk>.
;
; Several bugs exist in the KERNAL ROM in the routines that implement X-line
; handshaking. This wedge replaces the OPEN and NMI routines with code that fixes
; these issues.
;
;***********************************************************************************;
;***********************************************************************************;


; KERNAL zero page

STATUS	= $90			; I/O status byte
LDTND	= $98			; open file count
INBIT	= $A7			; tape write leader count/block count/RS-232 input bit
BITCI	= $A8			; tape error flags/tape long word marker/RS-232 input bit count
RINONE	= $A9			; tape dipole count/tape medium word marker/RS-232 start bit flag
RIDATA	= $AA			; tape input status/tape sync status/RS-232 byte assembly
RIPRTY	= $AB			; tape leader counter/tape read checksum/RS-232 parity bit
BITTS	= $B4			; transmitter bit count out
NXTBIT	= $B5			; transmitter next bit to be sent
RODATA	= $B6			; transmitter byte buffer/disassembly location
FNLEN	= $B7			; file name length
LA	= $B8			; logical file
SA	= $B9			; secondary address
FA	= $BA			; current device number
FNADR	= $BB			; file name pointer
ROPRTY	= $BD			; tape write byte/RS-232 parity byte
RIBUF	= $F7			; RS-232 Rx pointer
ROBUF	= $F9			; RS-232 Tx pointer

LAT	= $0259			; .. to $0262 logical file table
FAT	= $0263			; .. to $026C device number table
SAT	= $026D			; .. to $0276 secondary address table
M51CTR	= $0293			; pseudo 6551 control register. the first character of
					; the OPEN RS-232 filename will be stored here
					; bit	function
					; ---	--------
					;  7	2 stop bits/1 stop bit
					; 65	word length
					; ---	-----------
					; 00	8 bits
					; 01	7 bits
					; 10	6 bits
					; 11	5 bits
					;  4	unused
					; 3210	baud rate
					; ----	---------
					; 0000	user rate *
					; 0001	   50
					; 0010	   75
					; 0011	  110
					; 0100	  134.5
					; 0101	  150
					; 0110	  300
					; 0111	  600
					; 1000	 1200
					; 1001	 1800
					; 1010	 2400
					; 1011	 3600
					; 1100	 4800 *
					; 1101	 7200 *
					; 1110	 9600 *
					; 1111	19200 * * = not implemented
M51CDR	= $0294			; pseudo 6551 command register. the second character of
					; the OPEN RS-232 filename will be stored here
					; bit	function
					; ---	--------
					; 765	parity
					; ---	------
					; xx0	disabled
					; 001	odd
					; 011	even
					; 101	mark
					; 111	space
					;  4	duplex half/full
					;  3	unused
					;  2	unused
					;  1	unused
					;  0	handshake - X line/3 line
RSSTAT	= $0297			; RS-232 status register
					; bit	function
					; ---	--------
					;  7	break
					;  6	no DSR detected
					;  5	unused
					;  4	no CTS detected
					;  3	unused
					;  2	Rx buffer overrun
					;  1	framing error
					;  0	parity error
BITNUM	= $0298			; number of bits to be sent/received
BAUDOF	= $0299			; time of one bit cell
RIDBE	= $029B			; index to Rx buffer end
RIDBS	= $029C			; index to Rx buffer start
RODBS	= $029D			; index to Tx buffer start
RODBE	= $029E			; index to Tx buffer end

NMINV	= $0318			; NMI vector
IOPEN	= $031A			; KERNAL vector - open a logical file

VIA1PB		= $9110		; VIA 1 DRB
					; bit	function
					; ---	--------
					;  7	DSR in
					;  6	CTS in
					;  5	unused
					;  4	DCD in
					;  3	RI in
					;  2	DTR out
					;  1	RTS out
					;  0	data in
VIA1PA1		= $9111		; VIA 1 DRA
					; bit	function
					; ---	--------
					;  7	serial ATN out
					;  6	cassette switch
					;  5	joystick fire, light pen
					;  4	joystick left, paddle X fire
					;  3	joystick down
					;  2	joystick up
					;  1	serial DATA in
					;  0	serial CLK in
VIA1DDRB	= $9112		; VIA 1 DDRB
VIA1T1CL	= $9114		; VIA 1 T1C_l
VIA1T2CL	= $9118		; VIA 1 T2C_l
VIA1T2CH	= $9119		; VIA 1 T2C_h
VIA1PCR		= $911C		; VIA 1 PCR
					; bit	function
					; ---	--------
					; 765	CB2 Tx RS-232 data
					;  4	CB1 Rx RS-232 data
					; 321	CA2 cassette motor control
					;  0	CA1 [RESTORE] key

; The status bit is a not normal flag. It goes high if both an interrupt flag in the IFR
; and the corresponding enable bit in the IER are set. It can be cleared only by clearing
; all the active flags in the IFR or disabling all active interrupts in the IER.

VIA1IFR		= $911D		; VIA 1 IFR
					; bit	function		cleared by
					; ---	--------		----------
					;  7	interrupt status	clearing all enabled interrupts
					;  6	T1 interrupt		read T1C_l, write T1C_h
					;  5	T2 interrupt		read T2C_l, write T2C_h
					;  4	CB1 transition		read or write port B
					;  3	CB2 transition		read or write port B
					;  2	8 shifts done		read or write the shift register
					;  1	CA1 transition		read or write port A
					;  0	CA2 transition		read or write port A

; If enable/disable bit is a zero during a write to this register, each 1 in bits 0-6
; clears the corresponding bit in the IER. If this bit is a one during a write to this
; register, each 1 in bits 0-6 will set the corresponding IER bit.

VIA1IER		= $911E		; VIA 1 IER
					; bit	function
					; ---	--------
					;  7	enable/disable
					;  6	T1 interrupt
					;  5	T2 interrupt
					;  4	CB1 transition
					;  3	CB2 transition
					;  2	8 shifts done
					;  1	CA1 transition
					;  0	CA2 transition

XROMWARM	= $A002		; autostart ROM break entry vector

	; BASIC entrypoints
WARMST	= $C002			; BASIC warm start entry point

	; KERNAL entrypoints
INITSK	= $E518			; initialize hardware
RSCPTBIT = $F027		; compute bit count
FNDFLNO	= $F3CF			; find file
FUDTIM	= $F734			; increment real time clock
FE_2MNYF = $F77E		; display too many files
FE_ALOPN = $F781		; display file already open
FE_NTINP = $F78D		; display not input file
CHKAUTO	= $FD3F			; scan for autostart ROM at $A000
FRESTOR	= $FD52			; restore default I/O vectors
INITVIA	= $FDF9			; initialize I/O registers
BAUDTBL	= $FF5C			; baud rate tables
STOP	= $FFE1			; scan the stop key

*	=	$A000

	JMP RSPATCH
	NOP
	JMP RSUNPATCH
	NOP
	JMP RSREADST
#ifdef DEBUG
	NOP
	JMP RECVTST
	NOP
	JMP SENDTST
#endif

ENABL	= $02A1

ORGOPEN	= $03FC
ORGNMI	= $03FE


;***********************************************************************************;
;
; initialize RS-232 fix wedge
;
; Setup the OPEN and NMI vectors to point to code that correctly handles X-line
; (hardware) flow control.

RSPATCH
	LDA IOPEN		; get original OPEN vector low byte
	STA ORGOPEN		; save original OPEN vector low byte
	LDA IOPEN+1		; get original OPEN vector high byte
	STA ORGOPEN+1		; save original OPEN vector high byte
	LDA NMINV		; get original NMI vector low byte
	STA ORGNMI		; save original NMI vector low byte
	LDA NMINV+1		; get original NMI vector high byte
	STA ORGNMI+1		; save original NMI vector high byte
RSVECT	
	LDA #<RFOPEN		; get replacement OPEN vector low byte
	STA IOPEN		; set OPEN vector low byte
	LDA #>RFOPEN		; get replacement OPEN vector high byte
	STA IOPEN+1		; set OPEN vector high byte
	LDA #<RFNMI		; get replacement NMI vector low byte
	STA NMINV		; set NMI vector low byte
	LDA #>RFNMI		; get replacement NMI vector high byte
	STA NMINV+1		; set NMI vector high byte
	RTS


;***********************************************************************************;
;
; remove RS-232 fix wedge
;
; Restore the OPEN and NMI vectors to point to their initial values.

RSUNPATCH
	LDA ORGOPEN
	STA NMINV
	LDA ORGOPEN+1
	STA NMINV+1
	LDA ORGOPEN
	STA IOPEN
	LDA ORGOPEN+1
	STA IOPEN+1
	RTS


;***********************************************************************************;
;
; read I/O status word

; This routine returns the current status of the I/O device in the accumulator. The
; routine is usually called after new communication to an I/O device. The routine
; will give information about device status, or errors that have occurred during the
; I/O operation.

RSREADST
	LDA	FA			; get device number
	CMP	#$02			; compare device with RS-232 device
	BNE	READIOST		; branch if not RS-232 device

					; get RS-232 device status
	LDA	RSSTAT			; read RS-232 status word
	PHA
	LDA	#$00			; clear .X
	STA	RSSTAT			; clear RS-232 status
	PLA
	RTS

READIOST
	LDA	STATUS			; read I/O status byte
	RTS


;***********************************************************************************;
;
; open a logical file

; This routine is used to open a logical file. Once the logical file is set up it
; can be used for input/output operations. Most of the I/O KERNAL routines call on
; this routine to create the logical files to operate on. No arguments need to be
; set up to use this routine, but both the SETLFS and SETNAM KERNAL routines must
; be called before using this routine.

RFOPEN
.(
	LDX LA			; get logical file
	BNE LABEL1		; branch if there is a file

	JMP FE_NTINP		; else do not input file error and return

LABEL1
	JSR FNDFLNO		; find file
	BNE LABEL2		; branch if file not found

	JMP FE_ALOPN		; else do file already open error and return

LABEL2
	LDX LDTND		; get open file count
	CPX #$0A		; compare with max
	BCC LABEL3		; branch if less

	JMP FE_2MNYF		; else do too many files error and return

LABEL3
	INC LDTND		; increment open file count
	LDA LA			; get logical file
	STA LAT,X		; save to logical file table
	LDA SA			; get secondary address
	ORA #$60		; OR with the OPEN CHANNEL command
	STA SA			; set secondary address
	STA SAT,X		; save to secondary address table
	LDA FA			; get device number
	STA FAT,X		; save to device number table
	BEQ LABEL4		; do ok exit if keyboard
	CMP #$02		; compare device with RS-232 device
	BEQ OPENRS
	JMP $F439		; use KERNAL routine for other devices

LABEL4
	CLC				; flag ok
	RTS	
.)


;***********************************************************************************;
;
; open RS-232

OPENRS
	LDA	#$06			; IIII IOOI, DTR and RTS only as outputs
	STA	VIA1DDRB		; set VIA 1 DDRB
	STA	VIA1PB			; set VIA 1 DRB, DTR and RTS high
	LDA	#$EE			; CB2 high, CB1 -ve edge, CA2 high, CA1 -ve edge
	STA	VIA1PCR			; set VIA 1 PCR
	LDY	#$00			; clear index
	STY	RSSTAT			; clear RS-232 status byte
LAB_F4D9
	CPY	FNLEN			; compare with file name length
	BEQ	LAB_F4E7		; exit loop if done

	LDA	(FNADR),Y		; get file name byte
	STA	M51CTR,Y		; copy to 6551 register set
	INY				; increment index
	CPY	#$04			; compare with $04
	BNE	LAB_F4D9		; loop if not to 4 yet

LAB_F4E7
	JSR	RSCPTBIT		; compute bit count
	STX	BITNUM			; save bit count
	LDA	M51CTR			; get pseudo 6551 control register
	AND	#$0F			; mask 0000 xxxx, baud rate
	ASL				; * 2
	TAX				; copy to index
	LDA	BAUDTBL-2,X		; get timer constant low byte
	ASL				; * 2
	TAY				; copy to .Y
	LDA	BAUDTBL-1,X		; get timer constant high byte
	ROL				; * 2
	PHA				; save it
	TYA				; get timer constant low byte back
	ADC	#$C8			; + 200, carry cleared by previous ROL
	STA	BAUDOF			; save bit cell time low byte
	PLA				; restore high byte
	ADC	#$00			; add carry
	STA	BAUDOF+1		; save bit cell time high byte
	LDA	M51CDR			; get pseudo 6551 command register
	LSR				; shift b0 into Cb
	BCC	LAB_F51B		; branch if 3 line interface

	LDA	VIA1PB			; get VIA 1 DRB

	ASL				; shift DSR into Cb
	BCS	LAB_F51B		; branch if DSR = 1

	JMP	RSMISSNG		; set DSR signal not present and return

LAB_F51B
	LDA	RIDBE			; get index to Rx buffer end
	STA	RIDBS			; set index to Rx buffer start, clear Rx buffer
	LDA	RODBE			; get index to Tx buffer end
	STA	RODBS			; set index to Tx buffer start, clear Tx buffer
	JSR	$FE75			; read the top of memory
	LDA	RIBUF+1			; get RS-232 Rx buffer pointer high byte
	BNE	LAB_F533		; branch if buffer already set

	DEY				; decrement top of memory high byte, 256 byte buffer
	STY	RIBUF+1			; set RS-232 Rx buffer pointer high byte
	STX	RIBUF			; set RS-232 Rx buffer pointer low byte
LAB_F533
	LDA	ROBUF+1			; get RS-232 Tx buffer pointer high byte
	BNE	LAB_F53C		; branch if buffer already set

	DEY				; decrement Rx buffer pointer high byte, 256 byte buffer
	STY	ROBUF+1			; set RS-232 Tx buffer pointer high byte
	STX	ROBUF			; set RS-232 Tx buffer pointer low byte
LAB_F53C
	SEC				; non-standard exit, Cb set
	LDA	#$F0			; non-standard exit, $F0 error code
	JMP	$FE7B			; set the top of memory and return


;***********************************************************************************;
;
; re-enable interrupts and exit

NMIDONE
	LDA	ENABL			; get enabled interrupts
	ORA	#$80			; enable interrupts
	STA	VIA1IER			; set VIA 1 IER
	PLA				; pull .Y
	TAY				; restore .Y
	PLA				; pull .X
	TAX				; restore .X
	PLA				; restore .A
	RTI


;***********************************************************************************;
;
; NMI handler

RFNMI
	PHA				; save .A
	TXA				; copy .X
	PHA				; save .X
	TYA				; copy .Y
	PHA				; save .Y
	LDA	VIA1IER			; get VIA 1 IER, enabled interrupts
	STA	ENABL			; save for re-enabling on exit
	LDA	#$7F			; disable all interrupts, clears b7 of IFR
	STA	VIA1IER			; set VIA 1 IER

PROCNMI
	LDA	VIA1IFR			; get VIA 1 IFR, pending interrupts
	AND	ENABL			; mask with enabled interrupts
	BEQ	NMIDONE			; no more pending interrupts

	TAY				; copy to .Y
	AND	#$40			; mask timer 1 interrupt
	BEQ	TESTRXTM		; branch if not timer 1 interrupt

					; was VIA 1 T1 interrupt
	LDA	#$CE			; CB2 low, CB1 -ve edge, CA2 high, CA1 -ve edge
	ORA	NXTBIT			; OR RS-232 next bit to send, sets CB2 high if set
	STA	VIA1PCR			; set VIA 1 PCR
	LDA	VIA1T1CL		; get VIA 1 T1C_l, clear interrupt
	TYA
	PHA
	JSR	RSNXTBIT		; RS-232 Tx NMI routine
	PLA
	TAY

TESTRXTM
	TYA				; get pending interrupts back
	AND	#$20			; mask timer 2 interrupt
	BEQ	TESTRXST		; branch if not timer 1 interrupt

					; was VIA 1 T2 interrupt
	LDA	VIA1PB			; get VIA 1 DRB
	AND	#$01			; mask RS-232 data in
	STA	INBIT			; save RS-232 input bit
	LDA	VIA1T2CL		; get VIA 1 T2C_l, clear interrupt
	SBC	#$16			; adjust by 22 cycles to cover time taken by the
					; six instructions needed to write to the latch
	ADC	BAUDOF			; add baud rate bit time low byte
	STA	VIA1T2CL		; set VIA 1 T2C_l
	LDA	VIA1T2CH		; get VIA 1 T2C_h
	ADC	BAUDOF+1		; add baud rate bit time high byte
	STA	VIA1T2CH		; set VIA 1 T2C_h
	TYA
	PHA
	JSR	RSINBIT			; RS-232 Rx
	PLA
	TAY

TESTRXST
	TYA				; get pending interrupts back
	AND	#$10			; mask CB1 interrupt
	BEQ	RESTORE			; branch if not CB1 interrupt

					; was CB1 interrupt
	LDA	M51CTR			; get pseudo 6551 control register
	AND	#$0F			; clear non baud bits
	ASL				; *2, 2 bytes per baud count
	TAX				; copy to index
	LDA	BAUDTBL-2,X		; get baud count low byte
	STA	VIA1T2CL		; set VIA 1 T2C_l
	LDA	BAUDTBL-1,X		; get baud count high byte
	STA	VIA1T2CH		; set VIA 1 T2C_h
	LDA	VIA1PB			; read VIA 1 DRB, clear interrupt flag
	LDA	ENABL			; get enabled interrupts
	ORA	#$20			; enable T2 interrupt
	AND	#$EF			; mask CB1 interrupt
	STA	ENABL			; set enabled interrupts
	LDA	BITNUM			; get number of bits to be sent/received
	STA	BITCI			; save RS-232 input bit count

RESTORE
	TYA				; get pending interrupts back
	AND	#$02			; mask CA1 interrupt
	BEQ	PROCNMI			; repeat interrupt processing

					; else was [RESTORE] key ..
	BIT	VIA1PA1			; test VIA 1 DRA, clear CA1 interrupt
	JSR	FUDTIM			; increment the real time clock
	JSR	STOP			; scan stop key
	BEQ	DOSTOP			; if [STOP] perform reset
	JMP	PROCNMI			; if not [STOP] repeat interrupt processing

DOSTOP
	JSR	FRESTOR			; restore default I/O vectors
	JSR	RSVECT			; restore our vectors
	JSR	INITVIA			; initialise I/O registers
	JSR	INITSK			; initialise hardware
	JMP	(WARMST)		; do BASIC break entry


;***********************************************************************************;
;
; RS-232 Tx NMI routine

RSNXTBIT
	LDA	BITTS			; get RS-232 bit count
	BEQ	RSNXTBYT		; if zero go setup next RS-232 Tx byte and return

	BMI	RSSTOPS			; if -ve go do stop bit(s)

					; else bit count is non zero and +ve
	LSR	RODATA			; shift RS-232 output byte buffer
	LDX	#$00			; set $00 for bit = 0
	BCC	LAB_EFB0		; branch if bit was 0

	DEX				; set $FF for bit = 1
LAB_EFB0
	TXA				; copy bit to .A
	EOR	ROPRTY			; XOR with RS-232 parity byte
	STA	ROPRTY			; save RS-232 parity byte
	DEC	BITTS			; decrement RS-232 bit count
	BEQ	RSPRTY			; if RS-232 bit count now zero go do parity bit

; save bit and exit

LAB_EFB9
	TXA				; copy bit to .A
	AND	#$20			; mask for CB2 control bit
	STA	NXTBIT			; save RS-232 next bit to send
	RTS

; do RS-232 parity bit, enters with RS-232 bit count = 0

RSPRTY
	LDA	#$20			; mask 00x0 0000, parity enable bit
	BIT	M51CDR			; test pseudo 6551 command register
	BEQ	LAB_EFDA		; branch if parity disabled

	BMI	LAB_EFE4		; branch if fixed mark or space parity

	BVS	LAB_EFDE		; branch if even parity

					; else odd parity
	LDA	ROPRTY			; get RS-232 parity byte
	BNE	LAB_EFCF		; if parity not zero leave parity bit = 0

LAB_EFCE
	DEX				; make parity bit = 1
LAB_EFCF
	DEC	BITTS			; decrement RS-232 bit count, 1 stop bit
	LDA	M51CTR			; get pseudo 6551 control register
	BPL	LAB_EFB9		; if 1 stop bit save parity bit and exit

					; else two stop bits ..
	DEC	BITTS			; decrement RS-232 bit count, 2 stop bits
	BNE	LAB_EFB9		; save bit and exit, branch always

					; parity is disabled so the parity bit becomes the first,
					; and possibly only, stop bit. to do this increment the bit
					; count which effectively decrements the stop bit count.
LAB_EFDA
	INC	BITTS			; increment RS-232 bit count, = -1 stop bit
	BNE	LAB_EFCE		; set stop bit = 1 and exit

					; do even parity
LAB_EFDE
	LDA	ROPRTY			; get RS-232 parity byte
	BEQ	LAB_EFCF		; if parity zero leave parity bit = 0

	BNE	LAB_EFCE		; else make parity bit = 1, branch always

					; fixed mark or space parity
LAB_EFE4
	BVS	LAB_EFCF		; if fixed space parity leave parity bit = 0

	BVC	LAB_EFCE		; else fixed mark parity make parity bit = 1, branch always

; decrement stop bit count, set stop bit = 1 and exit. $FF is one stop bit, $FE is two
; stop bits

RSSTOPS
	INC	BITTS			; decrement RS-232 bit count
	LDX	#$FF			; set stop bit = 1
	BNE	LAB_EFB9		; save stop bit and exit, branch always

; setup next RS-232 Tx byte

RSNXTBYT
	LDA	M51CDR			; get 6551 pseudo command register
	LSR				; handshake bit into Cb
	BCC	LAB_EFFB		; branch if 3 line interface

	BIT	VIA1PB			; test VIA 1 DRB

	BPL	RSMISSNG		; if DSR = 0 set DSR signal not present and exit

	BVC	LAB_F019		; if CTS = 0 set CTS signal not present and exit

					; was 3 line interface
LAB_EFFB
	LDA	#$00			; clear .A
	STA	ROPRTY			; clear RS-232 parity byte
	STA	NXTBIT			; clear RS-232 next bit to send
	LDX	BITNUM			; get number of bits to be sent/received
	STX	BITTS			; set RS-232 bit count
	LDY	RODBS			; get index to Tx buffer start
	CPY	RODBE			; compare with index to Tx buffer end
	BEQ	LAB_F021		; if all done go disable T1 interrupt and return

	LDA	(ROBUF),Y		; else get byte from buffer
	STA	RODATA			; save to RS-232 output byte buffer
	INC	RODBS			; increment index to Tx buffer start
	RTS


;***********************************************************************************;
;
;## exit or quit
; set DSR signal not present

RSMISSNG
	LDA	#$40			; set DSR signal not present
	.byte	$2C			; makes next line BIT $10A9

; set CTS signal not present

LAB_F019
	LDA	#$10			; set CTS signal not present
	ORA	RSSTAT			; OR with RS-232 status register
	STA	RSSTAT			; save RS-232 status register

; disable T1 interrupt

LAB_F021
	LDA	ENABL			; get enabled interrupts
	AND	#$BF			; mask T1 interrupt
	STA	ENABL			; set enabled interrupts
	RTS


;***********************************************************************************;
;
; RS-232 Rx NMI

RSINBIT
	LDX	RINONE			; get RS-232 start bit check flag
	BNE	RSSTRBIT		; branch if no start bit received

	DEC	BITCI			; decrement RS-232 input bit count
	BEQ	RSINBYTE		; if the byte is complete go add it to the buffer

	BMI	LAB_F04D		; determine if all stop bits have been received

	LDA	INBIT			; get RS-232 input bit
	EOR	RIPRTY			; XOR with RS-232 parity bit
	STA	RIPRTY			; save in RS-232 parity bit
	LSR	INBIT			; shift RS-232 input bit into Cb
	ROR	RIDATA			; shift Cb into RS-232 byte assembly
LAB_F04A
	RTS


;***********************************************************************************;
;
; determine if all stop bits have been received

RSSTPBIT
	DEC	BITCI			; decrement RS-232 input bit count
LAB_F04D
	LDA	INBIT			; get RS-232 input bit
	BEQ	LAB_F0B3		; branch if bit clear

	LDA	M51CTR			; get pseudo 6551 control register
	ASL				; shift stop bits into Cb
	LDA	#$01			; one stop bit
	ADC	BITCI			; add stop bits to RS-232 input bit count
	BNE	LAB_F04A		; branch always


;***********************************************************************************;
;
; prepare to receive next input byte

RSPREPIN
	LDA	ENABL			; get enabled interrupts
	ORA	#$10			; enable CB1 interrupt, Rx data bit transition
	AND	#$DF			; mask T2 interrupt
	STA	ENABL			; set enabled interrupts
	LDA	#$90			;
	STA	RINONE			; set RS-232 start bit check flag, no start bit received
	RTS


;***********************************************************************************;
;
; no RS-232 start bit received

RSSTRBIT
	LDA	INBIT			; get RS-232 input bit
	BNE	RSPREPIN		; branch if bit set
	STA	RINONE			; set RS-232 start bit check flag, start bit received
	RTS


;***********************************************************************************;
;
; put byte into receive buffer

RSINBYTE
	LDY	RIDBE			; get index to Rx buffer end
	INY				; increment index
	CPY	RIDBS			; compare with index to Rx buffer start
	BEQ	RSOVERUN		; if buffer full go do Rx overrun error

	STY	RIDBE			; save index to Rx buffer end
	DEY				; decrement index
	LDA	RIDATA			; get assembled byte
	LDX	BITNUM			; get bit count
LAB_F081
	CPX	#$09			; compare with byte + stop
	BEQ	LAB_F089		; branch if all nine bits received

	LSR				; else shift byte
	INX				; increment bit count
	BNE	LAB_F081		; loop, branch always

LAB_F089
	STA	(RIBUF),Y		; save received byte to RS-232 Rx buffer
	LDA	#$20			; mask 00x0 0000, parity enable bit
	BIT	M51CDR			; test pseudo 6551 command register
	BEQ	RSSTPBIT		; branch if parity disabled

	BMI	LAB_F04A		; exit if mark or space parity

	LDA	INBIT			; get RS-232 input bit
	EOR	RIPRTY			; XOR with RS-232 parity bit
	BEQ	RSPRTYER		; branch if parity error

	BVS	LAB_F04A		; exit if even parity

	.byte	$2C			; makes next line BIT $AB50
RSPRTYER
	BVC	LAB_F04A		; exit if odd parity

	LDA	#$01			; set Rx parity error
	.byte	$2C			; makes next line BIT $04A9

RSOVERUN
	LDA	#$04			; set Rx overrun error
	.byte	$2C			; makes next line BIT $80A9

RSBREAK
	LDA	#$80			; Rx break error
	.byte	$2C			; makes next line BIT $02A9

RSFRAMER
	LDA	#$02			; Rx frame error
	ORA	RSSTAT			; OR with RS-232 status byte
	STA	RSSTAT			; save RS-232 status byte
	JMP	RSPREPIN		; prepare to receive next input byte

LAB_F0B3
	LDA	RIDATA			; get assembled byte
	BNE	RSFRAMER		; if not break do frame error

	BEQ	RSBREAK			; else do break error, branch always


#ifdef DEBUG

;***********************************************************************************;

DFLTN	= $99			; input device number
DFLTO	= $9A			; output device number

RSOPNOUT = $F0BC
RSOPNIN	= $F116
SETNAM	= $FFBD
CLRCHN	= $FFCC
CHRIN	= $FFCF
CHROUT	= $FFD2

RECVTST
	JSR	TSTOPEN
	JSR	RSOPNIN
	LDA	#2
	STA	DFLTN

RCVLOOP
	JSR	CHRIN
	CMP	#$20
	BCC	CTRLCHR

	LDX	LASTCHR
	BEQ	NOCHECK
	TAX
	DEX
	CPX	LASTCHR
	BNE	MISMTCH

NOCHECK
	JSR	CHROUT
	STA	LASTCHR
	BCC	CHKSTOP

CTRLCHR
	TAX
	BEQ	CHKSTOP
	LDA	#$0D
	JSR	CHROUT
	LDA	#0
	STA	LASTCHR

CHKSTOP
	JSR	STOP
	BNE	RCVLOOP
	JMP	CLRCHN

MISMTCH
	JSR	CHROUT
	LDA	#0
	STA	LASTCHR
	LDX	$900F
	INX
	TXA
	AND	#%00000111
	STA	$FF
	LDA	$900F
	AND	#%11111000
	ORA	$FF
	STA	$900F
	JMP	CHKSTOP

SENDTST
	JSR	TSTOPEN
	JSR	RSOPNOUT
	LDA	#2
	STA	DFLTO
	LDX	#'A'

SNDLOOP
	TXA
	JSR	CHROUT
	INX
	CMP	#'Z'
	BCC	SNDLOOP
	LDA	#13
	JSR	CHROUT
	LDA	#10
	JSR	CHROUT
	LDX	#'A'
	JSR	STOP
	BNE	SNDLOOP
	JMP	CLRCHN

M51NAME	.WORD	0, 0
LASTCHR	.BYT	0

	; Open RS-232 channel, 6551 control register in .A
TSTOPEN
	STA	M51NAME
	LDY	#0
	STY	M51NAME+1
	STY	LASTCHR
	LDA	#2
	LDX	#<M51NAME
	LDY	#>M51NAME
	JSR	SETNAM
	JMP	OPENRS

#endif
